/*

  This function will conduct SExtractor photometry for a given image.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <gsl/gsl_statistics.h>
#include <gsl/gsl_sort.h>

#include <sys/types.h> // for getpid()
#include <unistd.h> // also for getpid() and unlink() ... 

#include "limits.h"
#include "ident.h"
#include "guess_saturation_limit.h"

int find_catalog_in_vast_images_catalogs_log(char *fitsfilename, char *catalogfilename){
 char fitsfilename_to_test[FILENAME_LENGTH];
 FILE *f;
 f=fopen("vast_images_catalogs.log","r");
 if(f==NULL){
  strcpy(catalogfilename,"image00000.cat");
  return 1; // not only this image has not been processed, even "vast_images_catalogs.log" is not created yet!
 }
 int found=0;
 while(-1<fscanf(f,"%s %s",catalogfilename,fitsfilename_to_test))
  if(0==strcmp(fitsfilename_to_test,fitsfilename)){found=1;break;}
 fclose(f);
 if( found==0 ){
  strcpy(catalogfilename,"image00000.cat");
  return 1; // it is possible that image00000.cat is referring to another image, so we'll recompute...
 }
 /* Check if the catalog already exist */
 f=fopen(catalogfilename,"r");
 if(f==NULL)
  return 1;
 else
  fclose(f);
 return 0;
}

//double autodetect_aperture(char *fitsfilename, char *output_sextractor_catalog, int force_recompute, int param_P, double fixed_aperture, int verbose, double X_im_size, double Y_im_size){
double autodetect_aperture(char *fitsfilename, char *output_sextractor_catalog, int force_recompute, int param_P, double fixed_aperture, double X_im_size, double Y_im_size, int guess_saturation_limit_operation_mode){
         
	char command[1024];
        double *A;
	int i = 0;
	double median_A;
	double APERTURE;
	int sextractor_flag;


	char aperture_filename[FILENAME_LENGTH]; // this file simply stores the aperture size
	FILE *aperture_file;
	
        //int number_of_cpu_cores=N_FORK; // default 

        int N_bad_regions = 0;
        double *X1;
        double *Y1;
        double *X2;
        double *Y2;

        int pid=getpid();
	char sextractor_catalog_filename[FILENAME_LENGTH];
	char psf_filename[FILENAME_LENGTH];
	
	int good_stars_in_the_catalog=0;
	int all_stars_in_the_catalog=0;
        
        double star_x, star_y;
        
        char saturation_limitsextractor_cl_parameter_string[FILENAME_LENGTH];
        char gain_sextractor_cl_parameter_string[FILENAME_LENGTH];
        char flag_image_sextractor_cl_parameter_string[FILENAME_LENGTH];
        char flag_image_filename[FILENAME_LENGTH];
        int is_flag_image_used=2; // 2 - guess by default, 1 - always use the flag image, 0 - never use the flag image
                                  // The decision will also be stored in this variable: 1 - use the flag image, 0 - don't use it
        int external_flag;
        char external_flag_string[256];
        
        char sextractor_catalog_string[MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT];
        
        /* Check if the file has already been processed */
        if(0==find_catalog_in_vast_images_catalogs_log( fitsfilename, output_sextractor_catalog)){
         sprintf(aperture_filename,"%s.aperture",output_sextractor_catalog);
         aperture_file=fopen(aperture_filename,"r");
         if(NULL==aperture_file){fprintf(stderr,"ERROR: cannot open %s\n",aperture_filename);return 99.0;}
         fscanf(aperture_file,"%lf\n",&APERTURE);
         fclose(aperture_file);
         if(0==force_recompute){
          return APERTURE;
         }
        }
        sprintf(aperture_filename,"%s.aperture",output_sextractor_catalog);

        X1=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( X1==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for X1\n");exit(1);}
        Y1=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( Y1==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for Y1\n");exit(1);}
        X2=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( X2==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for X2\n");exit(1);}
        Y2=malloc(MAX_NUMBER_OF_BAD_REGIONS_ON_CCD*sizeof(double));
        if( Y2==NULL ){fprintf(stderr,"ERROR: in autodetect_aperture() can't allocate memory for Y2\n");exit(1);}
        
        read_bad_lst(X1, Y1, X2, Y2, &N_bad_regions);
      
	A=malloc(MAX_NUMBER_OF_STARS*sizeof(double));
	if( A==NULL ){fprintf(stderr,"ERROR: out of memory in function autodetect_aperture() A !!!\n");exit(1);}

        //if(param_P == 1)fprintf(stderr," --------------------------------- \n");
        
        // Check if we need a flag image
        check_if_we_need_flag_image( fitsfilename, flag_image_sextractor_cl_parameter_string, &is_flag_image_used, flag_image_filename);

        // Guess saturation limit for the given image
        if( 0!=guess_saturation_limit( fitsfilename, saturation_limitsextractor_cl_parameter_string, guess_saturation_limit_operation_mode) ){
         fprintf(stderr,"An ERROR ocurred while trying to guess image saturation limit for %s\n",fitsfilename);
         return 99.0;
        }

        // Guess gain for the given image
        if( 0!=guess_gain( fitsfilename, gain_sextractor_cl_parameter_string, 2) ){
         fprintf(stderr,"An ERROR ocurred while trying to guess CCD gain for %s\n",fitsfilename);
         return 99.0;
        }

	/* Set fixed aperture size if we whant to use it */
	if( fixed_aperture!=0.0 ){
	 APERTURE=fixed_aperture;
	}
	else{
         /* Calculate best aperture size from seeing */
	 sprintf(sextractor_catalog_filename,"autodetect_aper_%d.cat",pid);
	 if( is_flag_image_used==1 )
          sprintf(command, "sex -c default.sex %s%s%s -PARAMETERS_NAME autodetect_aper_flag.param -CATALOG_NAME %s  %s", gain_sextractor_cl_parameter_string, saturation_limitsextractor_cl_parameter_string, flag_image_sextractor_cl_parameter_string, sextractor_catalog_filename,fitsfilename);
	 else
	  sprintf(command, "sex -c default.sex %s%s -PARAMETERS_NAME autodetect_aper.param -CATALOG_NAME %s  %s", gain_sextractor_cl_parameter_string, saturation_limitsextractor_cl_parameter_string, sextractor_catalog_filename,fitsfilename);
	 fprintf(stderr, "%s\n", command);
	 system(command);
	 FILE *catalog;
	 catalog = fopen(sextractor_catalog_filename, "r");
	 if(catalog==NULL){fprintf(stderr,"ERROR: cannot open %s\n",sextractor_catalog_filename);return 99.0;}
	 //while(-1 < fscanf(catalog, "%lf %d %lf %lf", &A[i], &sextractor_flag, &star_x, &star_y)) {
         while( NULL!=fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,catalog) ){
          external_flag_string[0]='\0'; // reset, just in case
          //fprintf(stderr,"__#%s#__\n",sextractor_catalog_string);
          if(4>sscanf(sextractor_catalog_string, "%lf %d %lf %lf %[^\t\n]", &A[i], &sextractor_flag, &star_x, &star_y, external_flag_string))continue;
          //fprintf(stderr,"#%s#\n",external_flag_string);
          if( strlen(external_flag_string)>0 ){
           if( 1!=sscanf(external_flag_string,"%d",&external_flag) ){
            external_flag=0; // no external flag image used
           }
           //else
           // fprintf(stderr,"OhYEAHBaby external_flag=%d\n",external_flag);
          }
          else
           external_flag=0; // no external flag image used
	  all_stars_in_the_catalog++;
	  //if( external_flag==0 && sextractor_flag==0 && A[i]>0.5 && A[i]>FWHM_MIN && 0==exclude_region( X1, Y1, X2, Y2, N_bad_regions, star_x, star_y, 5.0) && star_x>FRAME_EDGE_INDENT_PIXELS && star_y>FRAME_EDGE_INDENT_PIXELS && fabs(star_x-X_im_size)>FRAME_EDGE_INDENT_PIXELS && fabs(star_y-Y_im_size)>FRAME_EDGE_INDENT_PIXELS )
	  if( external_flag==0 && sextractor_flag==0 && A[i]>FWHM_MIN && 0==exclude_region( X1, Y1, X2, Y2, N_bad_regions, star_x, star_y, 5.0) && star_x>FRAME_EDGE_INDENT_PIXELS && star_y>FRAME_EDGE_INDENT_PIXELS && fabs(star_x-X_im_size)>FRAME_EDGE_INDENT_PIXELS && fabs(star_y-Y_im_size)>FRAME_EDGE_INDENT_PIXELS ){
	   i++;good_stars_in_the_catalog++;
          }
	  if( i>=MAX_NUMBER_OF_STARS ){
           fprintf(stderr, "Oops!!! Too many stars!\nChange string \"#define MAX_NUMBER_OF_STARS %d\" in src/limits.h file and recompile the program by running \"make\".\n", MAX_NUMBER_OF_STARS);
           exit(1);
          } 
         }
	 // If most of the stars are flagged out - use flagged stars to choose aperture size
	 if( good_stars_in_the_catalog<0.5*all_stars_in_the_catalog ){
          fseek(catalog,0,SEEK_SET); // go back to the beginning of the lightcurve file
          i=0; // reset the counter that we use later
          good_stars_in_the_catalog=0;
	  //while( -1 < fscanf(catalog, "%lf %d %lf %lf", &A[i], &sextractor_flag, &star_x, &star_y) ){
	  while( NULL!=fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,catalog) ){
           external_flag_string[0]='\0'; // reset, just in case
           if(4>sscanf(sextractor_catalog_string, "%lf %d %lf %lf %[^\t\n]", &A[i], &sextractor_flag, &star_x, &star_y,external_flag_string))continue;
           //fprintf(stderr,"#%s#\n",external_flag_string);
           if( strlen(external_flag_string)>0 ){
            if( 1!=sscanf(external_flag_string,"%d",&external_flag) ){
             external_flag=0; // no external flag image used
            }
           }
           else
            external_flag=0; // no external flag image used
            
           all_stars_in_the_catalog++;
           //if( external_flag==0 && A[i]>0.5 && A[i]>FWHM_MIN && 0==exclude_region( X1, Y1, X2, Y2, N_bad_regions, star_x, star_y, 5.0) && star_x>FRAME_EDGE_INDENT_PIXELS && star_y>FRAME_EDGE_INDENT_PIXELS && fabs(star_x-X_im_size)>FRAME_EDGE_INDENT_PIXELS && fabs(star_y-Y_im_size)>FRAME_EDGE_INDENT_PIXELS )i++;good_stars_in_the_catalog++;
           if( external_flag==0 && A[i]>FWHM_MIN && 0==exclude_region( X1, Y1, X2, Y2, N_bad_regions, star_x, star_y, 5.0) && star_x>FRAME_EDGE_INDENT_PIXELS && star_y>FRAME_EDGE_INDENT_PIXELS && fabs(star_x-X_im_size)>FRAME_EDGE_INDENT_PIXELS && fabs(star_y-Y_im_size)>FRAME_EDGE_INDENT_PIXELS ){i++;good_stars_in_the_catalog++;}
           if( i>=MAX_NUMBER_OF_STARS ){
            fprintf(stderr, "Oops!!! Too many stars!\nChange string \"#define MAX_NUMBER_OF_STARS %d\" in src/limits.h file and recompile the program by running \"make\".\n",MAX_NUMBER_OF_STARS);
            exit(1);
           } 
	  }
          
	 } // if( good_stars_in_the_catalog<0.5*all_stars_in_the_catalog ){
	 // close the catalog file
	 fclose(catalog);

         // remove the SExtractor catalog from the 1st pass with wich we measured sizes of stars, we don't need it anymore
         if(0!=unlink(sextractor_catalog_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",sextractor_catalog_filename);
          
	 gsl_sort(A,1,i);
	 median_A=gsl_stats_median_from_sorted_data(A,1,i);
	 APERTURE = median_A * CONST;
	} //if( fixed_aperture!=0.0 )

        // Save aperture information
        aperture_file=fopen(aperture_filename,"w");
        if(aperture_file==NULL){fprintf(stderr,"ERROR: cannot open for writing %s\n",aperture_filename);return 99.0;}
        fprintf(aperture_file,"%.1lf",APERTURE);
        fclose(aperture_file);

        // Check if the aperture is suspiciously small
        if(APERTURE<1.0){
         fprintf(stderr,"\nWARNING: the estimated aperture is unrealistically small: %.1lf<1.0 pix (determined from %d stars).\nThis may happen if all stars are flagged as bad ones by SExtractor.\nCheck SExtractor settings in default.sex, especially that SATUR_LEVEL is not too low.\n\n",APERTURE,i);
         return APERTURE;
        }
	
	if (param_P == 0) {
               if( is_flag_image_used==1 )
		sprintf(command, "sex %s%s%s -PARAMETERS_NAME default_flag.param -PHOT_APERTURES %.1lf -CATALOG_NAME %s %s", gain_sextractor_cl_parameter_string, saturation_limitsextractor_cl_parameter_string, flag_image_sextractor_cl_parameter_string, APERTURE, output_sextractor_catalog, fitsfilename);
               else
		sprintf(command, "sex %s%s -PARAMETERS_NAME default.param -PHOT_APERTURES %.1lf -CATALOG_NAME %s %s", gain_sextractor_cl_parameter_string, saturation_limitsextractor_cl_parameter_string, APERTURE, output_sextractor_catalog, fitsfilename);
               fprintf(stderr, "%s\n", command);
               system(command);
	}
	if (param_P == 1) {

                char psfex_param_filename[512];
                char psfex_XML_check_filename[512];
                char psfex_log_entry_filename[512];

                sprintf(psfex_param_filename,"%s.psfex_param",output_sextractor_catalog);
                sprintf(psfex_XML_check_filename,"%s.psfex_check.xml",output_sextractor_catalog);
                sprintf(psfex_log_entry_filename,"%s.psfex_log",output_sextractor_catalog);
		sprintf(command, "echo \"VIGNET(%.0lf,%.0lf)\" > %s", 2.0*APERTURE+0.5,2.0*APERTURE+0.5, psfex_param_filename);
		system(command);
		sprintf(command, "echo \"XWIN_IMAGE\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"YWIN_IMAGE\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUX_RADIUS\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUX_MAX\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUX_APER(1)\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLUXERR_APER(1)\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"ELONGATION\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"FLAGS\" >> %s", psfex_param_filename);
		system(command);
		sprintf(command, "echo \"SNR_WIN\" >> %s", psfex_param_filename);
		system(command);


	        //sprintf(sextractor_catalog_filename,"psfex_input_%d.cat",pid);
	        sprintf(sextractor_catalog_filename,"%s.psfex_input_cat",output_sextractor_catalog);
	        //sprintf(psf_filename,"psfex_input_%d.psf",pid);
	        sprintf(psf_filename,"%s.psf",output_sextractor_catalog);
		sprintf(command, "sex -c bright_star_blend_check_3.0.sex %s%s -PARAMETERS_NAME %s -CATALOG_TYPE FITS_LDAC -CATALOG_NAME %s -PHOT_APERTURES %.1lf %s", gain_sextractor_cl_parameter_string, saturation_limitsextractor_cl_parameter_string, psfex_param_filename, sextractor_catalog_filename, APERTURE, fitsfilename);
		fprintf(stderr, "%s\n", command);
		system(command);
		//if(0!=unlink(psfex_param_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",psfex_param_filename);
		
		//sprintf(command, "psfex -c default.psfex -NTHREADS 1 -SAMPLE_FWHMRANGE %.1lf,%.1lf -XML_NAME %s %s &> %s", 0.5*APERTURE/2.2528, 1.3*APERTURE/2.2528, psfex_XML_check_filename,  sextractor_catalog_filename, psfex_log_entry_filename);
		sprintf(command, "psfex -c default.psfex -NTHREADS 1 -SAMPLE_FWHMRANGE %.2lf,%.2lf -XML_NAME %s %s &> %s", 0.3*APERTURE/2.2528, 1.3*APERTURE/2.2528, psfex_XML_check_filename,  sextractor_catalog_filename, psfex_log_entry_filename);
		//sprintf(command, "psfex -c default.psfex -NTHREADS 1 -SAMPLE_FWHMRANGE %.1lf,%.1lf -XML_NAME %s %s &> %s", 0.5*APERTURE/2.2528, 1.5*APERTURE/2.2528, psfex_XML_check_filename,  sextractor_catalog_filename, psfex_log_entry_filename);
		fprintf(stderr, "%s\n", command);
		system(command);
		//if(0!=unlink(sextractor_catalog_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",sextractor_catalog_filename);

		//number_of_cpu_cores=get_number_of_cpu_cores();

		//sprintf(command, "sex -c default.sex -PARAMETERS_NAME psfex_sextractor_2nd_pass.param -NTHREADS %d -PSF_NAME %s -PHOT_APERTURES %.1lf -CATALOG_NAME %s %s", number_of_cpu_cores, psf_filename, APERTURE, output_sextractor_catalog, fitsfilename);
		if( is_flag_image_used==1 )
		 sprintf(command, "sex -c default.sex %s%s%s -PARAMETERS_NAME psfex_sextractor_2nd_pass_flag.param -PSF_NMAX 1 -PSF_NAME %s -PHOT_APERTURES %.1lf -CATALOG_NAME %s %s", gain_sextractor_cl_parameter_string, saturation_limitsextractor_cl_parameter_string, flag_image_sextractor_cl_parameter_string, psf_filename, APERTURE, output_sextractor_catalog, fitsfilename);
		else
		 sprintf(command, "sex -c default.sex %s%s -PARAMETERS_NAME psfex_sextractor_2nd_pass.param -PSF_NMAX 1 -PSF_NAME %s -PHOT_APERTURES %.1lf -CATALOG_NAME %s %s", gain_sextractor_cl_parameter_string, saturation_limitsextractor_cl_parameter_string, psf_filename, APERTURE, output_sextractor_catalog, fitsfilename);
		fprintf(stderr, "%s\n", command);
		system(command);
		//if(0!=unlink(psf_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",psf_filename);

	}
	
	free(A);
	
	free(X1);
	free(Y1);
	free(X2);
	free(Y2);
	
	// Remove flag file to save disk space (if it was used)
	if( is_flag_image_used==1 ){
	 if(0!=unlink(flag_image_filename))fprintf(stderr,"WARNING! Cannot delete temporary file %s\n",flag_image_filename);
	}
	
	return APERTURE;
}
