/*

  For each star, this routine will find a check star with the most similar lightcurve 
  and investigate the remaining residuals.
  
*/

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>


#include <gsl/gsl_statistics_float.h>
#include <gsl/gsl_sort_float.h>
#include <gsl/gsl_errno.h>

#include "limits.h"

#include "lightcurve_io.h"

void write_fake_log_file(double *jd, int *Nobs){
 int i;
 FILE *logfile;
 fprintf(stderr,"Writing fake vast_image_details.log ... ");
 logfile=fopen("vast_image_details.log","w");
 for(i=0;i<(*Nobs);i++)
  fprintf(logfile,"JD= %.5lf\n",jd[i]);
 fclose(logfile);
 fprintf(stderr,"done\n");
 return;
}

void get_dates_from_lightcurve_files(double *jd, int *Nobs){
 DIR *dp;
 struct dirent *ep;
 FILE *lightcurvefile;
 double _jd,mag,merr,x,y,app;
 char string[FILENAME_LENGTH];
 int i,date_found;
 
 (*Nobs)=0;
 
 dp = opendir ("./");
 if (dp != NULL){
  fprintf(stderr,"Extracting list of Julian Days from lightcurves... ");
  while( ep = readdir(dp) ){
   if( strlen(ep->d_name)<8 )continue; // make sure the filename is not too short for the following tests
   if( ep->d_name[0]=='o' && ep->d_name[1]=='u' && ep->d_name[2]=='t' && ep->d_name[strlen(ep->d_name)-1]=='t' && ep->d_name[strlen(ep->d_name)-2]=='a' && ep->d_name[strlen(ep->d_name)-3]=='d' ){ 
    lightcurvefile=fopen(ep->d_name,"r");
    if( NULL==lightcurvefile ){
     fprintf(stderr,"ERROR: Can't open file %s\n",ep->d_name);
     exit(1);
    }
    while(-1<fscanf(lightcurvefile,"%lf %lf %lf %lf %lf %lf %s",&_jd,&mag,&merr,&x,&y,&app,string)){
     date_found=0;
     for(i=0;i<(*Nobs);i++){
      if( _jd==jd[i] ){
       date_found=1;
       break;
      }
     }
     if( date_found==0 ){
      jd[(*Nobs)]=_jd;
      (*Nobs)+=1;
     }
    }
    fclose(lightcurvefile);
   }
  }
  (void) closedir (dp);
  fprintf(stderr,"done\n");
 }
 else
  perror("Couldn't open the directory\n");
 
 /* Write a fake log file so we don't need to read all the lightcurves next time */
 write_fake_log_file( jd, Nobs);
       
 return;
}

void get_dates(double *jd, int *Nobs){
 FILE *vastlogfile;
 char str[MAX_LOG_STR_LENGTH];
 char jd_str[MAX_LOG_STR_LENGTH];
 unsigned int i,j,k;
 (*Nobs)=0;
 vastlogfile=fopen("vast_image_details.log","r");
 if( NULL==vastlogfile ){
  fprintf(stderr,"WARNING: Can't open vast_image_details.log\n");
  get_dates_from_lightcurve_files( jd, Nobs);  
 }
 else{
  while( NULL!=fgets( str, MAX_LOG_STR_LENGTH, vastlogfile) ){
   for(i=0;i<strlen(str)-3;i++)
    if( str[i]=='J' && str[i+1]=='D' && str[i+2]=='=' ){
     for(j=i+4,k=0; str[j]!=' ';j++,k++){
      jd_str[k]=str[j];
     }
     jd[(*Nobs)]=atof(jd_str);
     break;
    }
   (*Nobs)+=1; 
  }
  fclose(vastlogfile);
  fprintf(stderr,"Total number of observations (from log file) %d\n",(*Nobs));
  return;
 }
 fprintf(stderr,"Total number of observations %d\n",(*Nobs));
 return;
}

int main(){
 FILE *lightcurvefile;

 FILE *datafile;
 char data_m_sigma_line[DATA_PARSER_C_MAX_STR_LENGTH];

 int Nobs,Nstars,i,j,k;

 char **star_numbers;
 double *jd;

 float **mag_err;
 float **r;

 float *data;
 //float mean,median,sigma,sum1,sum2;
 float mean,median;
 char lightcurvefilename[DATA_PARSER_C_MAX_STR_LENGTH];
 char star_number_string[FILENAME_LENGTH];

 double djd;
 float dmag,dmerr,x,y,app;
 char string[FILENAME_LENGTH];

 float sum_sq_res,best_sum_sq_res;
 int best_k;
 
 float *mag_diff;
 float *diff_err;
 float Chi2;

 int n_good_1,n_good_2,n_good_common;

 float *median_arr;

 // Count stars we want to process 
 Nstars=0;
 datafile=fopen("data.m_sigma","r");
 if( NULL==datafile ){fprintf(stderr,"ERROR! Can't open file data.m_sigma\n");return 1;}
 while( NULL!=fgets(data_m_sigma_line,DATA_PARSER_C_MAX_STR_LENGTH,datafile) )Nstars++;
 fclose(datafile);
 fprintf(stderr,"Number of stars in data.m_sigma %d\n",Nstars);

 star_numbers=malloc(Nstars*sizeof(char *));
 for(i=Nstars;i--;){
  star_numbers[i]=malloc(OUTFILENAME_LENGTH*sizeof(char));
 }

 // Read the log file
 jd=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
 get_dates( jd, &Nobs);
 
 // Allocate memory 
 mag_err=malloc(Nstars*sizeof(float*));
 if( mag_err==NULL ){
  fprintf(stderr, "out of memory (mag_err)\n");
  exit(1);
 }
 r=malloc(Nstars*sizeof(float*));
 if( r==NULL ){
  fprintf(stderr, "out of memory (r)\n");
  exit(1);
 }
 
 median_arr=malloc(Nstars*sizeof(float));

 data=malloc(Nstars*Nobs*sizeof(float));
 if( data==NULL ){fprintf(stderr, "out of memory (data)\n");exit(1);}
 
 mag_diff=malloc(Nobs*sizeof(float));
 diff_err=malloc(Nobs*sizeof(float));
 
 //for(i=0;i<Nstars;i++){
 for(i=Nstars;i--;){
  mag_err[i]=malloc(Nobs*sizeof(float));
  r[i]=malloc(Nobs*sizeof(float)); // is it correct ?????
  if( r[i] == NULL || mag_err[i]==NULL ){
   fprintf(stderr, "out of memory (i=%d)\n",i);
   exit(1);
  }
 }
 
 //for(i=0;i<Nstars;i++)
 for(i=Nstars;i--;)
  //for(j=0;j<Nobs;j++)
  for(j=Nobs;j--;){
   r[i][j]=0.0;
   mag_err[i][j]=0.0;
  }


 // Read the data 
 i=j=0;
 datafile=fopen("data.m_sigma","r");
 if( NULL==datafile ){fprintf(stderr,"ERROR! Can't open file sysrem_input_star_list.lst\n");exit(1);}
 while( -1<fscanf(datafile,"%f %f %f %f %s",&mean,&mean,&mean,&mean,lightcurvefilename) ){
  // Get star number from the lightcurve file name 
  for(k=3;k<(int)strlen(lightcurvefilename);k++){
   star_number_string[k-3]=lightcurvefilename[k];
   if( lightcurvefilename[k]=='.' ){
    star_number_string[k-3]='\0';
    break;
   }
  }
  strcpy(star_numbers[i],star_number_string);
  lightcurvefile=fopen(lightcurvefilename,"r");
  if( NULL==lightcurvefile ){fprintf(stderr,"ERROR: Can't read file %s\n",lightcurvefilename);exit(1);}
  while(-1<fscanf(lightcurvefile,"%lf %f %f %f %f %f %s",&djd,&dmag,&dmerr,&x,&y,&app,string)){
   // Find which j is corresponding to the current JD
   for(k=0;k<Nobs;k++){
    if( fabs(jd[k]-djd)<=0.00001 ){ // 0.8 sec
    //if( fabs(jd[k]-djd)<=0.0001 ){ // 8 sec
     j=k;
     r[i][j]=dmag;
     mag_err[i][j]=dmerr;
     break;
    }
   }
  }
  fclose(lightcurvefile);                     
  i++;
 } 
 fclose(datafile);

 // Do the actual work 
 fprintf(stderr,"Computing average magnitudes... ");

 // For each star compute median magnitude and subtract it from all measurements 
 //for(i=0;i<Nstars;i++){
 for(i=Nstars;i--;){
  k=0;
  //for(j=0;j<Nobs;j++){
  for(j=Nobs;j--;){
   if( r[i][j]!=0.0 ){
    data[k]=r[i][j];k++;
   }
  }
  gsl_sort_float(data,1,k);
  median=gsl_stats_float_median_from_sorted_data(data,1,k);
  median_arr[i]=median;
  //for(j=0;j<Nobs;j++){
  for(j=Nobs;j--;){
   if( r[i][j]!=0.0 ){
    r[i][j]=r[i][j]-median;
   }
  }
 }
 fprintf(stderr,"done\nFinding the most similar check star for each one...\n");

 datafile=fopen("vast_autocandidates.log","w");
 if( NULL==datafile ){fprintf(stderr,"ERROR! Can't open file vast_autocandidates.log\n");return 1;}

 // For each star
 for(i=Nstars;i--;){
  best_sum_sq_res=99999999999999999.9;
  for(n_good_1=0,j=Nobs;j--;)
   if( r[i][j]!=0.0 )
    n_good_1++;
  for(k=Nstars;k--;){
   if( i==k )continue;
   for(n_good_2=0,j=Nobs;j--;)
    if( r[k][j]!=0.0 )
     n_good_2++;
   if( abs(n_good_1-n_good_2)>10 )continue;
   if( fabsf(median_arr[i]-median_arr[k])>1.0 )continue;
   for(n_good_common=0,j=Nobs;j--;)
    if( r[i][j]!=0.0 && r[k][j]!=0.0 )
     n_good_common++;
   if( (float)n_good_common<0.9*(float)n_good_1 )continue;
   //fprintf(stderr,"median_arr[i]=%f median_arr[k]=%f  n_good_1=%d n_good_2=%d\n",median_arr[i],median_arr[k],n_good_1,n_good_2);
   sum_sq_res=0.0;
   for(j=Nobs;j--;){
    if( r[i][j]!=0.0 && r[k][j]!=0.0 ){
     sum_sq_res+=(r[i][j]-r[k][j])*(r[i][j]-r[k][j]);
    }
   }
   if( sum_sq_res<best_sum_sq_res ){best_sum_sq_res=sum_sq_res;best_k=k;}
  }
  //fprintf(stderr,"out%s.dat out%s.dat\n", star_numbers[i],star_numbers[best_k]);
  
  for(Chi2=0.0,k=0,j=Nobs;j--;){
   if( r[i][j]!=0.0 && r[best_k][j]!=0.0 ){
    mag_diff[k]=r[i][j]-r[best_k][j];
    diff_err[k]=sqrtf(mag_err[i][j]*mag_err[i][j]+mag_err[best_k][j]*mag_err[best_k][j]);
    Chi2+=mag_diff[k]*mag_diff[k]/(diff_err[k]*diff_err[k]);
    k++;
   }
  }
  Chi2=Chi2/(float)(k-1);

  //fprintf(stderr,"out%s.dat out%s.dat %lg\n", star_numbers[i],star_numbers[best_k],Chi2);
  
  fprintf(stdout,"%.4lf %lg\n", median_arr[i],Chi2);

  //if( Chi2>5.0 )fprintf(stderr,"out%s.dat\n", star_numbers[i]);
  if( Chi2>5.0 )fprintf(datafile,"out%s.dat\n", star_numbers[i]);
 }
 fclose(datafile);

 free(median_arr);

 free(diff_err);
 free(mag_diff);


 for(i=Nstars;i--;){
  free(star_numbers[i]);
  free(mag_err[i]);
  free(r[i]);
 }
 free(star_numbers);
 free(mag_err);
 free(r);
 
 free(jd);

 return 0;
}

/*
//int main(int argc, char **argv){
int main(){
 FILE *lightcurvefile;   
 FILE *outlightcurvefile;
    
 float **mag_err;
 float **r;
 
 float *c;
 float *a;

 float *old_c;
 float *old_a;
 
 double *jd;
 
 int i,j,iter;
 
 long k;
 
 double djd;
 float dmag,dmerr,x,y,app;
 char string[FILENAME_LENGTH];

 char star_number_string[FILENAME_LENGTH];
 
 int Nobs;
 int Nstars;

 float *data;
 float mean,median,sigma,sum1,sum2;

 char system_command_str[1024];

 FILE *datafile;
 char lightcurvefilename[DATA_PARSER_C_MAX_STR_LENGTH];

 int stop_iterations=0;

 int *bad_stars;
 char **star_numbers;

 float tmpfloat; // for faster computation

 // Protection against strange free() crashes
 //setenv("MALLOC_CHECK_", "0", 1);

 // If there is no input star list - make it 
 system("lib/select_sysrem_input_star_list");

 // Count stars we want to process 
 Nstars=0;
 datafile=fopen("sysrem_input_star_list.lst","r");
 if( NULL==datafile ){fprintf(stderr,"ERROR! Can't open file sysrem_input_star_list.lst\n");exit(1);}
 while( -1<fscanf(datafile,"%f %f %f %f %s",&mean,&mean,&mean,&mean,lightcurvefilename) )Nstars++;
 fclose(datafile);
 fprintf(stderr,"Number of stars in sysrem_input_star_list.lst %d\n",Nstars);
 if( Nstars<100 ){
  fprintf(stderr,"Too few stars!\n");
  exit(1);
 }

 bad_stars=malloc(Nstars*sizeof(int));
 star_numbers=malloc(Nstars*sizeof(char *));
 //for(i=0;i<Nstars;i++){
 for(i=Nstars;i--;){
  star_numbers[i]=malloc(80*sizeof(char));
  bad_stars[i]=0;
 }
 
 // Read the log file
 jd=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
 get_dates( jd, &Nobs);
 
 // Allocate memory 
 mag_err=malloc(Nstars*sizeof(float*));
 if( mag_err==NULL ){
  fprintf(stderr, "out of memory (mag_err)\n");
  exit(1);
 }
 r=malloc(Nstars*sizeof(float*));
 if( r==NULL ){
  fprintf(stderr, "out of memory (r)\n");
  exit(1);
 }

 data=malloc(Nstars*Nobs*sizeof(float));
 if( data==NULL ){fprintf(stderr, "out of memory (data)\n");exit(1);}
 
 //for(i=0;i<Nstars;i++){
 for(i=Nstars;i--;){
  mag_err[i]=malloc(Nobs*sizeof(float));
  r[i]=malloc(Nobs*sizeof(float)); // is it correct ?????
  if( r[i] == NULL || mag_err[i]==NULL ){
   fprintf(stderr, "out of memory (i=%d)\n",i);
   exit(1);
  }
 }
 
 //for(i=0;i<Nstars;i++)
 for(i=Nstars;i--;)
  //for(j=0;j<Nobs;j++)
  for(j=Nobs;j--;)
   r[i][j]=0.0;
  
 c=malloc(Nstars*sizeof(float));
 if( c==NULL ){fprintf(stderr, "out of memory (c)\n");exit(1);}
 a=malloc(Nobs*sizeof(float));
 if( a==NULL ){fprintf(stderr, "out of memory (a)\n");exit(1);}

 old_c=malloc(Nstars*sizeof(float));
 if( old_c==NULL ){fprintf(stderr, "out of memory (old_c)\n");exit(1);}
 old_a=malloc(Nobs*sizeof(float));
 if( old_a==NULL ){fprintf(stderr, "out of memory (old_a)\n");exit(1);}

 //for(i=0;i<Nstars;i++){
 for(i=Nstars;i--;){
  c[i]=0.1;
  old_c[i]=0.1;
 }
 
 //for(i=0;i<Nobs;i++){
 for(i=Nobs;i--;){
  a[i]=1.0;
  old_a[i]=1.0;
 }
 
 
 // Read the data 
 i=j=0;
 datafile=fopen("sysrem_input_star_list.lst","r");
 if( NULL==datafile ){fprintf(stderr,"ERROR! Can't open file sysrem_input_star_list.lst\n");exit(1);}
 while( -1<fscanf(datafile,"%f %f %f %f %s",&mean,&mean,&mean,&mean,lightcurvefilename) ){
  // Get star number from the lightcurve file name 
  for(k=3;k<(long)strlen(lightcurvefilename);k++){
   star_number_string[k-3]=lightcurvefilename[k];
   if( lightcurvefilename[k]=='.' ){
    star_number_string[k-3]='\0';
    break;
   }
  }
  strcpy(star_numbers[i],star_number_string);
  lightcurvefile=fopen(lightcurvefilename,"r");
  if( NULL==lightcurvefile ){fprintf(stderr,"ERROR: Can't read file %s\n",lightcurvefilename);exit(1);}
  while(-1<fscanf(lightcurvefile,"%lf %f %f %f %f %f %s",&djd,&dmag,&dmerr,&x,&y,&app,string)){
   // Find which j is corresponding to the current JD
   for(k=0;k<Nobs;k++){
    if( fabs(jd[k]-djd)<=0.00001 ){ // 0.8 sec
    //if( fabs(jd[k]-djd)<=0.0001 ){ // 8 sec
     j=k;
     r[i][j]=dmag;
     mag_err[i][j]=dmerr;
     break;
    }
   }
  }
  fclose(lightcurvefile);                     
  i++;
 } 
 fclose(datafile);

 // Do the actual work 
 fprintf(stderr,"Computing average magnitudes... ");

 // For each star compute median magnitude and subtract it from all measurements 
 //for(i=0;i<Nstars;i++){
 for(i=Nstars;i--;){
  k=0;
  //for(j=0;j<Nobs;j++){
  for(j=Nobs;j--;){
   if( r[i][j]!=0.0 ){
    data[k]=r[i][j];k++;
   }
  }
  gsl_sort_float(data,1,k);
  median=gsl_stats_float_median_from_sorted_data(data,1,k);
  //for(j=0;j<Nobs;j++){
  for(j=Nobs;j--;){
   if( r[i][j]!=0.0 ){
    r[i][j]=r[i][j]-median;
   }
  }
 }
 fprintf(stderr,"done\nStarting iterations...\n");

 // Iterative search for best c[i] and a[j]  
 for(iter=0;iter<NUMBER_OF_Ai_Ci_ITERATIONS;iter++){
  fprintf(stderr,"\riteration %4d",iter+1);
  //for(i=0;i<Nstars;i++){
  for(i=Nstars;i--;){
   sum1=sum2=0.0;
   for(j=0;j<Nobs;j++){
   //for(j=Nobs;j--;){
    if( r[i][j]!=0.0 ){
     tmpfloat=1.0f/(mag_err[i][j]*mag_err[i][j]);
     sum1+=r[i][j]*a[j]*tmpfloat;// /(mag_err[i][j]*mag_err[i][j]);
     sum2+=a[j]*a[j]*tmpfloat;// /(mag_err[i][j]*mag_err[i][j]);
    }
   }
   if( sum1!=0.0 && sum2!=0.0 ){
    old_c[i]=c[i];
    c[i]=sum1/sum2;
   }
  }

  //for(j=0;j<Nobs;j++){
  for(j=Nobs;j--;){
   sum1=sum2=0.0;
   //for(i=0;i<Nstars;i++){
   for(i=Nstars;i--;){
    if( r[i][j]!=0.0 ){
     tmpfloat=1.0f/(mag_err[i][j]*mag_err[i][j]);
     sum1+=r[i][j]*c[i]*tmpfloat;// /(mag_err[i][j]*mag_err[i][j]);
     sum2+=c[i]*c[i]*tmpfloat;// /(mag_err[i][j]*mag_err[i][j]);
    }
   }
   if( sum1!=0.0 && sum2!=0.0){
    old_a[j]=a[j];
    a[j]=sum1/sum2;
   }
  }

  // Should we stop now? 
  stop_iterations=1;
  //for(i=0;i<Nstars;i++){
  for(i=Nstars;i--;){
   if( fabsf(c[i]-old_c[i])>Ai_Ci_DIFFERENCE_TO_STOP_ITERATIONS ){
    stop_iterations=0;
    break;
   }
  }

  if( stop_iterations==1 ){
   //for(j=0;j<Nobs;j++){
   for(j=Nobs;j--;){
    if( fabsf(a[j]-old_a[j])>Ai_Ci_DIFFERENCE_TO_STOP_ITERATIONS ){
     stop_iterations=0;
     break;
    }
   }
  }
  if( stop_iterations==1 )break; // Stop iteretions if they make no difference
  
 }// Iterative search for best c[i] and a[j]


 fprintf(stderr,"\nRemoving outliers... ");

 // WTF?!?!? is it working?????

 // Check for obviously bad corrections 
 //for(j=0;j<Nobs;j++){
 for(j=Nobs;j--;){
  if( 0!=isnan(a[j]) ){fprintf(stderr,"a[%d]= %f\n",j,a[j]);exit(1);}
  k=0;
  //for(i=0;i<Nstars;i++){
  for(i=Nstars;i--;){
   if( r[i][j]!=0.0 ){
    data[k]=a[j]*c[i];
    k++;
   }
  }
  mean=gsl_stats_float_mean(data,1,k);
  sigma=gsl_stats_float_sd_m(data,1,k,mean);
  gsl_sort_float(data,1,k);
  median=gsl_stats_float_median_from_sorted_data(data,1,k);

  //for(i=0;i<Nstars;i++){
  for(i=Nstars;i--;){
   if( r[i][j]!=0.0 && fabsf(a[j]*c[i]-median)>3.0*sigma )bad_stars[i]=1;
  }
 }
 fprintf(stderr,"done\n");


 // 2nd pass 

 for(iter=0;iter<NUMBER_OF_Ai_Ci_ITERATIONS;iter++){
  fprintf(stderr,"\riteration %4d",iter+1);
  //for(i=0;i<Nstars;i++){
  for(i=Nstars;i--;){
   sum1=sum2=0.0;
   if( bad_stars[i]==0 ){
    //for(j=0;j<Nobs;j++){
    for(j=Nobs;j--;){
     if( r[i][j]!=0.0 ){
      tmpfloat=1.0f/(mag_err[i][j]*mag_err[i][j]);
      sum1+=r[i][j]*a[j]*tmpfloat;// /(mag_err[i][j]*mag_err[i][j]);
      sum2+=a[j]*a[j]*tmpfloat;// /(mag_err[i][j]*mag_err[i][j]);
     }
    }
   }
   if( sum1!=0.0 && sum2!=0.0 ){
    old_c[i]=c[i];
    c[i]=sum1/sum2;
   }
  }

  //for(j=0;j<Nobs;j++){
  for(j=Nobs;j--;){
   sum1=sum2=0.0;
   if( bad_stars[i]==0 ){
    //for(i=0;i<Nstars;i++){
    for(i=Nstars;i--;){
     if( r[i][j]!=0.0 ){
      tmpfloat=1.0f/(mag_err[i][j]*mag_err[i][j]);
      sum1+=r[i][j]*c[i]*tmpfloat; // /(mag_err[i][j]*mag_err[i][j]);
      sum2+=c[i]*c[i]*tmpfloat; // /(mag_err[i][j]*mag_err[i][j]);
     }
    }
   }
   if( sum1!=0.0 && sum2!=0.0){
    old_a[j]=a[j];
    a[j]=sum1/sum2;
   }
  }

  // Should we stop now?
  stop_iterations=1;
  //for(i=0;i<Nstars;i++){
  for(i=Nstars;i--;){
   if( fabsf(c[i]-old_c[i])>Ai_Ci_DIFFERENCE_TO_STOP_ITERATIONS ){
    stop_iterations=0;
    break;
   }
  }

  if( stop_iterations==1 ){
   //for(j=0;j<Nobs;j++){
   for(j=Nobs;j--;){
    if( fabsf(a[j]-old_a[j])>Ai_Ci_DIFFERENCE_TO_STOP_ITERATIONS ){
     stop_iterations=0;
     break;
    }
   }
  }

  if( stop_iterations==1 )break; // Stop iteretions if they make no difference

 }// Iterative search for best c[i] and a[j]


 fprintf(stderr,"\nRemoving outliers... ");
 // 2nd check for obviously bad corrections 
 for(j=0;j<Nobs;j++){
  if( 0!=isnan(a[j]) ){fprintf(stderr,"a[%d]= %f\n",j,a[j]);exit(1);}
  k=0;
  for(i=0;i<Nstars;i++){
   if( bad_stars[i]==0 )
    if( r[i][j]!=0.0 ){
     data[k]=a[j]*c[i];
     k++;
    }
  }
  mean=gsl_stats_float_mean(data,1,k);
  sigma=gsl_stats_float_sd_m(data,1,k,mean);
  gsl_sort_float(data,1,k);
  median=gsl_stats_float_median_from_sorted_data(data,1,k);

  for(i=0;i<Nstars;i++){
   //if( r[i][j]!=0.0 && fabsf(a[j]*c[i]-mean)>5*sigma )bad_stars[i]=1;
   if( r[i][j]!=0.0 && fabsf(a[j]*c[i]-median)>3*sigma )bad_stars[i]=1;
   if( r[i][j]!=0.0 && fabsf(a[j]*c[i]-median)>6*sigma )bad_stars[i]=2;
  }
 }
 fprintf(stderr,"done\n");

 // Apply corrections to lightcurves 
 fprintf(stderr,"Applying corrections... \n");
 for(i=0;i<Nstars;i++){
  sprintf(lightcurvefilename,"out%s.dat", star_numbers[i]);
  if( bad_stars[i]==0 ){
   lightcurvefile=fopen(lightcurvefilename,"r");
   if( NULL==lightcurvefile ){fprintf(stderr,"ERROR: Can't read file %s\n",lightcurvefilename);exit(1);}
   outlightcurvefile=fopen("TMP.dat","w");
   while(-1<fscanf(lightcurvefile,"%lf %f %f %f %f %f %s",&djd,&dmag,&dmerr,&x,&y,&app,string)){
    // Find which j is corresponding to the current JD
    for(k=0;k<Nobs;k++){
     if( fabs(jd[k]-djd)<=0.00001 ){ // 0.8 sec
     //if( fabs(jd[k]-djd)<=0.0001 ){ // 8 sec
      j=k;
      //fprintf(outlightcurvefile,"%.5lf %8.5f %.5f %8.3f %8.3f %4.lf %s\n",djd,dmag-c[i]*a[j],dmerr,x,y,app,string);
      write_lightcurve_point( outlightcurvefile, djd, (double)(dmag-c[i]*a[j]), (double)dmerr, (double)x, (double)y, (double)app, string);
      break;
     }
    }
   }
   fclose(outlightcurvefile);
   fclose(lightcurvefile);
   sprintf(system_command_str,"mv -f TMP.dat %s",lightcurvefilename);
   system(system_command_str);
  }
  else{
   fprintf(stderr,"Skip correction for %s",lightcurvefilename);
   if( bad_stars[i]==2 ){
    fprintf(stderr," removing it from sysrem_input_star_list.lst\n");
    sprintf(system_command_str,"grep -v %s sysrem_input_star_list.lst > TMP.dat && mv -f TMP.dat sysrem_input_star_list.lst",lightcurvefilename);
    system(system_command_str);
   }
   else
     fprintf(stderr,"\n");
  }
 } 
 fprintf(stderr,"done\n");
  
 // Print out some stats
 k=0;
 for(i=0;i<Nstars;i++){
  if( bad_stars[i]==0 ){
   for(j=0;j<Nobs;j++){
    if( r[i][j]!=0.0 ){
     data[k]=fabsf(a[j]*c[i]);
     k++;
    }
   }
  }
 }
 mean=gsl_stats_float_mean(data,1,k);
 sigma=gsl_stats_float_sd_m(data,1,k,mean);
 gsl_sort_float(data,1,k);
 median=gsl_stats_float_median_from_sorted_data(data,1,k);
 fprintf(stderr,"Mean correction %.6f +/-%.6f  (median=%lf)\n",mean,sigma,median);
       
       
 //!!!
 free(bad_stars);
 for(i=0;i<Nstars;i++){
  free(star_numbers[i]);
  free(mag_err[i]);
  free(r[i]);
 }
 free(star_numbers);
 free(old_c);
 free(old_a);

 // Free memory 
 free(mag_err);
 free(r);
 free(a);
 free(c);
 free(jd);
 free(data);

 change_number_of_sysrem_iterations_in_log_file();
 
// unsetenv("MALLOC_CHECK_");
 
 fprintf(stderr,"All done!  =)\n");
 
 return 0;
}

*/
