#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h> // for basename()
#include <sys/stat.h> // for stat(), also requires #include <sys/types.h> and #include <unistd.h>
#include <sys/types.h> // for stat()
#include <unistd.h> // for stat()

#include "fitsio.h"

#include "limits.h"

// defined in autodetect_aperture.c
// therefore any program that uses guess_saturation_limit() should be linked against autodetect_aperture.o
int find_catalog_in_vast_images_catalogs_log(char *fitsfilename, char *catalogfilename); 

// 1 - default gain value is found in default.sex
//     will alow the program to change it
// 0 - non-default gain value is found in default.sex
//     the program will use the value from default.sex
int check_gain_in_default_sex(double *out_gain_from_default_sex){
 double gain_in_default_sex;
 int i,j;
 char str1[256];
 FILE * f;
 f=fopen("default.sex","r");
 if(NULL==f)return 1;
 while( NULL!=fgets(str1, 256, f) ){
  str1[255]='\0'; // just in case
  if( str1[0]=='#' )continue;
  if( strlen(str1)<6 )continue;
  for(i=0;i<(int)strlen(str1)-6;i++){
   if( str1[i]=='#' )break; // go to next line
   if( str1[i]=='G' && str1[i+1]=='A' && str1[i+2]=='I' && str1[i+3]=='N' ){
    // this is our key
    // make sure there is no other stuff written after the gain value
    for(j=i+5;j<(int)strlen(str1)-6;j++){
     if( str1[j]=='#' ){str1[j]='\0';break;}
     if( str1[j]!='0' 
      && str1[j]!='1' 
      && str1[j]!='2' 
      && str1[j]!='3' 
      && str1[j]!='4' 
      && str1[j]!='5' 
      && str1[j]!='6' 
      && str1[j]!='7' 
      && str1[j]!='8' 
      && str1[j]!='9' 
      && str1[j]!='.' 
      && str1[j]!=' '
      && str1[j]!='\t' ){str1[j]='\0';break;}
    }
    // read the gain value
    sscanf(str1,"GAIN %lf",&gain_in_default_sex);
    //fprintf(stderr,"DERIVED GAIN %lf, #%s#\n",gain_in_default_sex,str1);
    (*out_gain_from_default_sex)=gain_in_default_sex;
    fclose(f);
    if( gain_in_default_sex==5.0 || gain_in_default_sex==0.0 ){
     return 0;
    }
    else{
     return 1;
    }
   }
  }
 }
 fclose(f); 
 return 1;
}


// Try to set the correct CCD gain from FITS image header
// operation_mode=0 // do nothing
// operation_mode=1 // force the use of gain guessed from images
// operation_mode=2 // try to be clever
int guess_gain(char * fitsfilename, char * resulting_sextractor_cl_parameter_string, int operation_mode){

 char str[256]; // for TSTRING type keys from FITS header

 fitsfile *fptr;  // FITS file pointer 
 int status = 0;  // CFITSIO status value MUST be initialized to zero! 
 int hdutype; //, naxis, ii;
 //long naxes[2], fpixel[2];
 //double *pix, minval = 1.E33, maxval = -1.E33;
 
 double guessed_gain,gain_from_fits_header;

 double exposure=0.0;
 
 //double saturation_level_from_default_sex=0.0;


 // Check the input
 if( operation_mode<0 || operation_mode>2 ){
  fprintf(stderr,"ERROR: unknown operation_mode=%d (should be 0, 1 or 2)\n",operation_mode);
  return 1;
 }

 // do nothing if we don't need to
 if( operation_mode==0 ){
  resulting_sextractor_cl_parameter_string[0]='\0';
  return 0;
 }


 // Try to play smart here: if a non-default gain value is specified in default.sex - use this one, don't try to guess
 if( operation_mode==2 ){
  if( 1==check_gain_in_default_sex(&guessed_gain) ){
   resulting_sextractor_cl_parameter_string[0]='\0';
   fprintf(stderr,"The gain value (GAIN=%.3lf) from default.sex will be used for image %s\nThe above guess is based on the non-default gain value specified in default.sex by user\n",guessed_gain,fitsfilename);
   return 0;
  }
 }

 if( !fits_open_image(&fptr, fitsfilename, READONLY, &status) ){
  if(fits_get_hdu_type(fptr, &hdutype, &status) || hdutype != IMAGE_HDU) { 
   fprintf(stderr,"Error: this program only works on images, not tables\n");
   return 1;
  }
  // Special case for HST images
  fits_read_key(fptr, TSTRING, "BUNIT", str, NULL, &status);
  if( status==0 ){
   gain_from_fits_header=-99.9;
   if( 0==strncmp(str,"count   ",8) )gain_from_fits_header=1.0; // Swift/UVOT
   if( 0==strncmp(str,"ELECTRONS",9) )gain_from_fits_header=1.0; // HST raw images
   if( 0==strncmp(str,"ELECTRONS/S",11) ){ // HST level 1 (drizzled/resampled images)
    fits_read_key(fptr, TDOUBLE, "EXPTIME", &exposure, NULL, &status);
    if( status==0 ){
     gain_from_fits_header=exposure; // Gain is equal to exposure time if this is a count rate image!
    }
   }
   // If we managed to recognize the BUNIT key...
   if( gain_from_fits_header!=-99.9 ){
    fits_close_file(fptr, &status); // close file
    guessed_gain=gain_from_fits_header;
    sprintf(resulting_sextractor_cl_parameter_string,"-GAIN %.3lf ",guessed_gain);
    fprintf(stderr,"The gain value (GAIN=%.3lf) is set based on the FITS header key BUNIT=%s of the image %s\n",guessed_gain,str,fitsfilename);
    return 0;
   }
  }
  status=0; // reset status, it is OK not to find this keyword
  
  // Normal GAIN keyword
  fits_read_key(fptr, TDOUBLE, "GAIN", &gain_from_fits_header, NULL, &status);
  if( status==0 ){
   fits_close_file(fptr, &status); // close file
   guessed_gain=gain_from_fits_header;
   sprintf(resulting_sextractor_cl_parameter_string,"-GAIN %.3lf ",guessed_gain);
   fprintf(stderr,"The gain value (GAIN=%.3lf) is obtained from the FITS header of the image %s\n",guessed_gain,fitsfilename);
   return 0;
  }
  else{
   status=0; // reset status, it's OK not to find the GAIN keyword in the header
   fits_close_file(fptr, &status); // close file
   fprintf(stderr,"WARNING: no gain value could be found for the image %s\nPlease specify it using GAIN keyword in default.sex or image FITS header.\n",fitsfilename);
   resulting_sextractor_cl_parameter_string[0]='\0';
   return 0;
  }
  fits_close_file(fptr, &status); // close file
 }

 if( status!=0 ){
  fits_report_error(stderr, status); // print any error message 
  return 1;
 }
 
 resulting_sextractor_cl_parameter_string[0]='\0';

 return 1; // we are not supposed to get here under the normal circumstances
}

// This function will count the number of zeroes in an image and if there are many - will create a flag image 
// for the SExtractor to flag-out stars near zero-leel pixels
int check_if_we_need_flag_image(char * fitsfilename, char * resulting_sextractor_cl_parameter_string, int * is_flag_image_used, char * flag_image_filename){

 fitsfile *fptr;  // FITS file pointer 
 int status = 0;  // CFITSIO status value MUST be initialized to zero! 
 int hdutype, naxis, ii;
 long naxes[2], totpix, fpixel[2];
 double *pix;
 long number_of_zeroes=0;
 char *flag;
 fitsfile *outfptr;  // FITS file pointer
 char outfilename[FILENAME_LENGTH];
 struct stat sb; // structure returned by stat() system call

 int anynul=0,nullval=0;
 
 long X,Y,X0,Y0;

 long j,k,l;
 
 // If user requests not to use the flag image by setting is_flag_image_used=0
 if( is_flag_image_used==0 ){
  // Nothing to do, we'll be fine even without a flag image
  (*is_flag_image_used)=0;
  resulting_sextractor_cl_parameter_string[0]='\0';
  flag_image_filename[0]='\0';
  return 0;
 }

 // If user requests to use the flag image, we don't need to guess
 if ( !fits_open_image(&fptr, fitsfilename, READONLY, &status) ){

   if (fits_get_hdu_type(fptr, &hdutype, &status) || hdutype != IMAGE_HDU) { 
     fprintf(stderr,"Error: this program only works on images, not tables\n");
     (*is_flag_image_used)=0; // just in case
     resulting_sextractor_cl_parameter_string[0]='\0'; // just in case
     flag_image_filename[0]='\0'; // just in case
     return 1;
   }
 
   fits_get_img_dim(fptr, &naxis, &status);
   fits_get_img_size(fptr, 2, naxes, &status);
 
   if (status || naxis != 2) { 
     fprintf(stderr,"Error: NAXIS = %d.  Only 2-D images are supported.\n", naxis);
     (*is_flag_image_used)=0; // just in case
     resulting_sextractor_cl_parameter_string[0]='\0'; // just in case
     flag_image_filename[0]='\0'; // just in case
     return 1;
   }
   pix = (double *) malloc(naxes[0] * sizeof(double)); /* memory for 1 row */
 
   if (pix == NULL) {
     fprintf(stderr,"Memory allocation error\n");
     (*is_flag_image_used)=0; // just in case
     resulting_sextractor_cl_parameter_string[0]='\0'; // just in case
     flag_image_filename[0]='\0'; // just in case
     return 1;
   }
 
   totpix = naxes[0] * naxes[1];
   fpixel[0] = 1;  /* read starting with first pixel in each row */
 
   /* process image one row at a time; increment row # in each loop */
   for (fpixel[1] = 1; fpixel[1] <= naxes[1]; fpixel[1]++)
   {  
      /* give starting pixel coordinate and number of pixels to read */
      if (fits_read_pix(fptr, TDOUBLE, fpixel, naxes[0],0, pix,0, &status))
         break;   /* jump out of loop on error */
 
      for (ii = 0; ii < naxes[0]; ii++) {
       if( pix[ii]==0.0 )number_of_zeroes++;
       if( pix[ii]==65535.0 )number_of_zeroes++;
       if( pix[ii]<MIN_PIX_VALUE )number_of_zeroes++;
       if( pix[ii]>MAX_PIX_VALUE )number_of_zeroes++;
      }
   }
   
   free(pix);
   fits_close_file(fptr, &status);

 } // if ( !fits_open_image(&fptr, fitsfilename, READONLY, &status) ){

 if( status!=0 ){
  fits_report_error(stderr, status); /* print any error message */
  (*is_flag_image_used)=0; // just in case
  resulting_sextractor_cl_parameter_string[0]='\0'; // just in case
  flag_image_filename[0]='\0'; // just in case
  return 1;
 }

 //fprintf(stderr,"\n\n\nFound %ld pixels with zero values out of %ld pixels (%lf\%)\n\n\n",number_of_zeroes,totpix,(double)number_of_zeroes/(double)totpix*100.0);

 if( (double)number_of_zeroes/(double)totpix<FRACTION_OF_ZERO_PIXEL_TO_USE_FLAG_IMG ){
  // Nothing to do, we'll be fine even without a flag image
  (*is_flag_image_used)=0;
  resulting_sextractor_cl_parameter_string[0]='\0';
  flag_image_filename[0]='\0';
  return 0;
 }
 
 // else ...
 
 // re-open the image

 if ( !fits_open_image(&fptr, fitsfilename, READONLY, &status) ){


//  strcpy(outfilename,basename(fitsfilename));
  find_catalog_in_vast_images_catalogs_log( fitsfilename, outfilename);
  outfilename[strlen(outfilename)-4]='\0';
  strcat(outfilename,".flag");

  // Remove the old flag image with this name if exist
  // Try to stat() the flag file
  if(0==stat(outfilename, &sb)){
   fprintf(stderr,"Hmm... Found an old flag file %s\n",outfilename);
   // Make sure this is not a directory or some funny file
   if( (sb.st_mode & S_IFMT) != S_IFREG ){
    fprintf(stderr,"%s is not a regular file or symlink!\nSomething is very-very wrong. Aborting computations.\n",outfilename);
    exit(1);
   }
   // If this is just an old file - delete it.
   fprintf(stderr,"Removing the old flag file %s\n",outfilename);
   unlink(outfilename);
  }

  // create the new empty output file if the above checks are OK 
  if( !fits_create_file(&outfptr, outfilename, &status) ){
   fits_create_img(outfptr, BYTE_IMG, 2, naxes, &status);

   pix = (double *) malloc( totpix * sizeof(double)); // memory for the input imae
   if (pix == NULL) {
     fprintf(stderr,"Memory allocation error\n");
     (*is_flag_image_used)=0; // just in case
     resulting_sextractor_cl_parameter_string[0]='\0'; // just in case
     flag_image_filename[0]='\0'; // just in case
     return 1;
   }
   flag = (char *) malloc( totpix * sizeof(char)); // memory for the flag image 
   if (flag == NULL) {
     fprintf(stderr,"Memory allocation error\n");
     (*is_flag_image_used)=0; // just in case
     resulting_sextractor_cl_parameter_string[0]='\0'; // just in case
     flag_image_filename[0]='\0'; // just in case
     return 1;
   }
   
   fits_read_img(fptr, TDOUBLE, 1, totpix,&nullval,pix, &anynul, &status);
   
   //-------------------------------------
   // array index to X Y
   // Y=1+(int)((float)i/(float)naxes[0]);
   // X=i+1-(Y-1)*naxes[0];
   //-------------------------------------
   // X Y to array index
   // i=X-1+(Y-1)*naxes[0];
   //-------------------------------------
   // reset flag values
   for (ii = 0; ii < totpix; ii++)flag[ii]=0;
   for (ii = 0; ii < totpix; ii++) {
    if( pix[ii]==0.0 || pix[ii]==65535.0 || pix[ii]<MIN_PIX_VALUE || pix[ii]>MAX_PIX_VALUE ){
     //flag[ii]=1;
     /// Mark as suspicious also 3 pixels around the bad one
     Y0=1+(long)((float)ii/(float)naxes[0]);
     X0=ii+1-(Y0-1)*naxes[0];
     for(j=-1*FLAG_N_PIXELS_AROUND_BAD_ONE;j<=FLAG_N_PIXELS_AROUND_BAD_ONE;j++){
      X=X0+j;
      if( X<1 )continue;
      if( X>naxes[0] )continue;
      for(k=-1*FLAG_N_PIXELS_AROUND_BAD_ONE;k<=FLAG_N_PIXELS_AROUND_BAD_ONE;k++){
       Y=Y0+k;
       if( Y<1 )continue;
       if( Y>naxes[1] )continue;
       l=X-1+(Y-1)*naxes[0];
       flag[l]=1;
      }
     }
     ///
    }
   }
   fits_write_img(outfptr, TBYTE, 1, totpix, flag, &status);
   
  /*
   // This is old version with row-by-row processing
   
   
   pix = (double *) malloc(naxes[0] * sizeof(double)); // memory for 1 row 
   if (pix == NULL) {
     fprintf(stderr,"Memory allocation error\n");
     return 1;
   }
   flag = (char *) malloc(naxes[0] * sizeof(char)); // memory for 1 row 
   if (flag == NULL) {
     fprintf(stderr,"Memory allocation error\n");
     return 1;
   }
 
   totpix = naxes[0] * naxes[1];
   fpixel[0] = 1;  // read starting with first pixel in each row
 
   // process image one row at a time; increment row # in each loop
   for (fpixel[1] = 1; fpixel[1] <= naxes[1]; fpixel[1]++){  
      // give starting pixel coordinate and number of pixels to read 
      if (fits_read_pix(fptr, TDOUBLE, fpixel, naxes[0],0, pix,0, &status))
         break;   // jump out of loop on error 
 
      for (ii = 0; ii < naxes[0]; ii++) {
       //fprintf(stderr,"%lf %d\n",pix[ii], status);
       if( pix[ii]==0.0 ){
        flag[ii]=(char)1;
        //fprintf(stderr,"AGAGA %d\n",(int)flag[ii]);
       }
       else{
        flag[ii]=(char)0;
        //fprintf(stderr,"OGOGOG %d\n",(int)flag[ii]);
       }
      }
      
      // expand the flagged area
      for(ii = 1; ii < naxes[0]-1; ii++){
       if( flag[ii-1]==1 ){flag[ii]=(char)1;ii=ii+2;continue;}
       if( flag[ii+1]==1 ){flag[ii]=(char)1;ii=ii+2;continue;}
      }
      
      fits_write_pix(outfptr, TBYTE, fpixel, naxes[0], flag, &status); // write new values to output image

   } // process image one row at a time; increment row # in each loop
   */
   
   fits_close_file(outfptr, &status);
   free(flag);
   free(pix);
  }
  fits_close_file(fptr, &status);

 }

 if( status!=0 ){
  fits_report_error(stderr, status); /* print any error message */
  (*is_flag_image_used)=0; // just in case
  resulting_sextractor_cl_parameter_string[0]='\0'; // just in case
  flag_image_filename[0]='\0'; // just in case
  return 1;
 }

 (*is_flag_image_used)=1;
 sprintf(resulting_sextractor_cl_parameter_string,"-FLAG_IMAGE %s ",outfilename);
 sprintf(flag_image_filename,"%s",outfilename);
 
 return 0;
}

// 1 - default saturation level value is found in default.sex
//     will alow the program to change it
// 0 - non-default saturation level value is found in default.sex
//     the program will use the value from default.sex
int check_saturation_limit_in_default_sex(double *out_saturation_level_from_default_sex){
 double satur_level_in_default_sex;
 int i,j;
 char str1[256];
 FILE * f;
 f=fopen("default.sex","r");
 if(NULL==f)return 1;
 while( NULL!=fgets(str1, 256, f) ){
  str1[255]='\0'; // just in case
  if( str1[0]=='#' )continue;
  if( strlen(str1)<13 )continue;
  for(i=0;i<(int)strlen(str1)-13;i++){
   if( str1[i]=='#' )break; // go to next line
   if( str1[i]=='S' && str1[i+1]=='A' && str1[i+2]=='T' && str1[i+3]=='U' && str1[i+4]=='R' && str1[i+5]=='_' && str1[i+6]=='L' && str1[i+7]=='E' && str1[i+8]=='V' && str1[i+9]=='E' && str1[i+10]=='L' ){
    // this is our key
    // make sure there is no other stuff written after the key value
    for(j=i+12;j<(int)strlen(str1)-13;j++){
     if( str1[j]=='#' ){str1[j]='\0';break;}
     if( str1[j]!='0' 
      && str1[j]!='1' 
      && str1[j]!='2' 
      && str1[j]!='3' 
      && str1[j]!='4' 
      && str1[j]!='5' 
      && str1[j]!='6' 
      && str1[j]!='7' 
      && str1[j]!='8' 
      && str1[j]!='9' 
      && str1[j]!='.' 
      && str1[j]!=' '
      && str1[j]!='\t' ){str1[j]='\0';break;}
    }
    sscanf(str1,"SATUR_LEVEL %lf",&satur_level_in_default_sex);
    (*out_saturation_level_from_default_sex)=satur_level_in_default_sex;
    fclose(f);
    if( satur_level_in_default_sex==60000.0 ){
     return 0;
    }
    else{
     return 1;
    }
   }
  }
 }
 fclose(f); 
 return 1;
}

// operation_mode=0 // do nothing
// operation_mode=1 // force the use of satuation limit guessed from images
// operation_mode=2 // try to be clever
int guess_saturation_limit(char * fitsfilename, char * resulting_sextractor_cl_parameter_string, int operation_mode){

 fitsfile *fptr;  // FITS file pointer 
 int status = 0;  // CFITSIO status value MUST be initialized to zero! 
 int hdutype, naxis, ii;
 long naxes[2], fpixel[2];
 double *pix, minval = 1.E33, maxval = -1.E33;
 
 double guessed_saturation_limit=0.0;

 double exposure=0.0;
 
 double saturation_level_from_default_sex=0.0;

 char str[256]; // for TSTRING type keys from FITS header

 // do nothing if we don't need to
 if( operation_mode==0 ){
  resulting_sextractor_cl_parameter_string[0]='\0';
  //strncpy(resulting_sextractor_cl_parameter_string,"",2);
  return 0;
 }

 // Check the input
 if( operation_mode<0 || operation_mode>2 ){
  fprintf(stderr,"ERROR: unknown operation_mode=%d (should be 0, 1 or 2)\n",operation_mode);
  return 1;
 }

 if ( !fits_open_image(&fptr, fitsfilename, READONLY, &status) ){

   if (fits_get_hdu_type(fptr, &hdutype, &status) || hdutype != IMAGE_HDU) { 
     fprintf(stderr,"Error: this program only works on images, not tables\n");
     return 1;
   }
 
   fits_get_img_dim(fptr, &naxis, &status);
   fits_get_img_size(fptr, 2, naxes, &status);
 
   if (status || naxis != 2) { 
     fprintf(stderr,"Error: NAXIS = %d.  Only 2-D images are supported.\n", naxis);
     return 1;
   }
   
   // Get exposure time
   fits_read_key(fptr, TDOUBLE, "EXPTIME", &exposure, NULL, &status);
   if( status==202 ){
    status = 0;
    fits_read_key(fptr, TDOUBLE, "EXPOSURE", &exposure, NULL, &status);
    if( status==202 ){
     exposure = 0.0;
     status = 0;
    }
   }
   
   // Try to play smart here: if the image is small, it is unlikely to have saturated stars on it
   // In that case, revert back to the default SATUR_LEVEL set by user in SExtractor configuration file default.sex
   if( operation_mode==2 ){
    if( 1==check_saturation_limit_in_default_sex(&saturation_level_from_default_sex) ){
     resulting_sextractor_cl_parameter_string[0]='\0';
     fprintf(stderr,"The value of saturation limit (SATUR_LEVEL=%.1lf) from default.sex will be used for image %s\nThe above guess is based on the non-default saturation level specified in default.sex by user\n",saturation_level_from_default_sex,fitsfilename);
     return 0;
    }
    if( naxes[0]<3000 || naxes[1]<3000 ){
     resulting_sextractor_cl_parameter_string[0]='\0';
     fprintf(stderr,"The value of saturation limit (SATUR_LEVEL=%.1lf) from default.sex will be used for image %s\nThe above guess is based on the small image size of %ldx%ld pix\n",saturation_level_from_default_sex,fitsfilename,naxes[0],naxes[1]);
     return 0;
    }
    if( exposure>0.0 && exposure<15.0 ){
     resulting_sextractor_cl_parameter_string[0]='\0';
     fprintf(stderr,"The value of saturation limit (SATUR_LEVEL=%.1lf) from default.sex will be used for image %s\nThe above guess is based on the short exposure time of %.1lf sec\n",saturation_level_from_default_sex,fitsfilename,exposure);
     return 0;
    }
   }
   //fprintf(stderr,"Guessing saturation limit for %s\n",fitsfilename);
 
   pix = (double *) malloc(naxes[0] * sizeof(double)); /* memory for 1 row */
 
   if (pix == NULL) {
     fprintf(stderr,"Memory allocation error\n");
     return 1;
   }
 
   //totpix = naxes[0] * naxes[1];
   fpixel[0] = 1;  /* read starting with first pixel in each row */
 
   /* process image one row at a time; increment row # in each loop */
   for (fpixel[1] = 1; fpixel[1] <= naxes[1]; fpixel[1]++)
   {  
      /* give starting pixel coordinate and number of pixels to read */
      if (fits_read_pix(fptr, TDOUBLE, fpixel, naxes[0],0, pix,0, &status))
         break;   /* jump out of loop on error */
 
      for (ii = 0; ii < naxes[0]; ii++) {
        //sum += pix[ii];                      /* accumlate sum */
        if (pix[ii] < minval) minval = pix[ii];  /* find min and  */
        if (pix[ii] > maxval) maxval = pix[ii];  /* max values    */
      }
   }
   
   // Special case of HST countrate images
   fits_read_key(fptr, TSTRING, "BUNIT", str, NULL, &status);
   if( status==0 ){
    if( 0==strncmp(str,"ELECTRONS/S",11) ){ // HST level 1 (drizzled/resampled images)
     guessed_saturation_limit=70000.0/exposure; // The maximum full well depth of the pixels on the WFC3/UVIS chips is ~72500 e-
    }
   }

   free(pix);
   fits_close_file(fptr, &status);
 }

 if( status!=0 ){
  fits_report_error(stderr, status); /* print any error message */
  return 1;
 }

 // if guessed_saturation_limit was not set above
 if( guessed_saturation_limit==0.0 )
  guessed_saturation_limit=maxval-SATURATION_LIMIT_INDENT*maxval;
 else
  guessed_saturation_limit=MIN( maxval-SATURATION_LIMIT_INDENT*maxval, guessed_saturation_limit); // take minimmum of the values based on the assumed well depth and on the actually found brightest pixel

 fprintf(stderr,"Guessing that the saturation limit for %s is %.2lf ADU\n",fitsfilename,guessed_saturation_limit);

 // Suggest saturation level based on image pixel values
 sprintf(resulting_sextractor_cl_parameter_string,"-SATUR_LEVEL %.2lf ",guessed_saturation_limit);

 return 0;
}
